function vk_HTMLProvider()
{
    this.vk_Log( 10, "vk_HTMLProvider", "()" );
}

vk_HTMLProvider.prototype.Init = function( on_complete, on_failure, timeout, base_url )
{
    this.vk_Log( 9, "vk_HTMLProvider", "Init()" );

    //       
    this.on_complete = on_complete;
    //       
    this.on_failure = on_failure;

    //   ,     
    this.timeout = timeout;
    //  url        HTTPRequest
    this.base_url = base_url;

    //        ,      
    this.response_status_handlers = new Array();

    // Request is in progress
    this.in_progress = false;

    this.request_params =
    {
        url: undefined,
        method: undefined,
        headers: undefined,
        body: undefined
    };
}

vk_HTMLProvider.prototype.HTTPRequest = function( url, method, headers, body )
{
    this.vk_Log( 8, "vk_HTMLProvider", "HTTPRequest(), url=" + url );

    if ( !url )
        url = "";
    if ( !method )
        method = "GET";
    if ( !headers )
        headers = new Object();
    if ( !body )
        body = null;

    this.request_params.url = url;
    this.request_params.method = method;
    this.request_params.headers = headers;
    this.request_params.body = body;

    //      ,    
    if ( this.in_progress )
    {
        this.vk_Log( 0, "vk_HTMLProvider", "HTTPRequest(): Error! Attempting to start new request while processing one!" );
        this.Abort();
    }

    vk_global.request_queue.AddRequest( this );
}

vk_HTMLProvider.prototype.HTTPRequestFinal = function()
{
    this.vk_Log( 8, "vk_HTMLProvider", "HTTPRequestFinal()" );

    if ( vk_engine.gecko )
    {
        this.request = new (Components.Constructor(
            "@mozilla.org/xmlextras/xmlhttprequest;1",
            Components.interfaces.nsIXMLHttpRequest ));
    }
    else
    {
        this.request = new XMLHttpRequest();
    }

    this.request.open( this.request_params.method, this.base_url + this.request_params.url, true ); // true -  

    //   HTTP    headers
    for ( var i in this.request_params.headers )
        this.request.setRequestHeader( i, this.request_params.headers[i] );

    // OnReadyStateChange        
    this.request.onreadystatechange = vk_callback( this, "OnReadyStateChange" );

    this.StartObserve_();

    //  , ..     OnReadyStateChange
    //this.request.onerror = vk_callback( this, "OnError" );
    //this.request.onload = vk_callback( this, "OnLoad" );
    //this.request.onprogress = vk_callback( this, "OnProgress" );

    //    
    if ( this.timeout >= 0 )
        this.timeout_timer = vk_setTimeout( vk_callback( this, "OnTimeout" ), this.timeout );
    else
        this.timeout_timer = undefined;

    this.in_progress = true;

    //  
    this.request.send( this.request_params.body );
}

vk_HTMLProvider.prototype.Abort = function()
{
    this.vk_Log( 8, "vk_HTMLProvider", "Abort()" );

    this.request.abort();

    if ( vk_engine.webkit )
    {
        this.StopObserve_();
        vk_global.request_queue.OnRequestFinished();
        this.on_failure();
    }
}

//  302 - redirect
vk_HTMLProvider.prototype.AddResponseStatusHandler = function( status, handler )
{
    this.vk_Log( 8, "vk_HTMLProvider", "AddResponseStatusHandler()" );

    // handler may be undefined or null
    this.response_status_handlers[status] = handler;
}

//      
vk_HTMLProvider.prototype.StartObserve_ = function()
{
    this.vk_Log( 8, "vk_HTMLProvider", "StartObserve_()" );

    if ( vk_engine.gecko )
    {
        this.observer_service = Components.classes[ "@mozilla.org/observer-service;1" ]
            .getService( Components.interfaces.nsIObserverService );

        this.observer_service.addObserver( this, "http-on-modify-request", false );
        this.observer_service.addObserver( this, "http-on-examine-response", false );
    }
}

//      
vk_HTMLProvider.prototype.StopObserve_ = function()
{
    this.vk_Log( 8, "vk_HTMLProvider", "StopObserve_()" );

    if ( vk_engine.gecko )
    {
        this.observer_service.removeObserver( this, "http-on-modify-request" );
        this.observer_service.removeObserver( this, "http-on-examine-response" );
    }
}

//       
vk_HTMLProvider.prototype.observe = function( subject, topic, data )
{
    this.vk_Log( 8, "vk_HTMLProvider", "observe(), subject=" + subject + ", topic=" + topic + ", data=" + data );

    var channel = subject.QueryInterface( Components.interfaces.nsIHttpChannel );

    if ( channel == this.request.channel )
    {
        if ( topic == "http-on-modify-request" )
        {
        }
        else if ( topic == "http-on-examine-response" )
        {
            this.vk_Log( 8, "vk_HTMLProvider", "observe(), Response status=" + this.request.status );

            //     ,   
            for ( var i in this.response_status_handlers )
            {
                if ( this.request.status == i )
                {
                    if
                    (
                        (this.response_status_handlers[i] != undefined) &&
                        (this.response_status_handlers[i] != null)
                    )
                    this.response_status_handlers[i]();
                }
            }
        }
    }
}

vk_HTMLProvider.prototype.OnReadyStateChange = function()
{
    this.vk_Log( 8, "vk_HTMLProvider", "OnReadyStateChange(), readyState=" + this.request.readyState );

    // readyState
    // 0 UNINITIALIZED open() has not been called yet.
    // 1 LOADING send() has not been called yet.
    // 2 LOADED send() has been called, headers and status are available.
    // 3 INTERACTIVE Downloading, responseText holds the partial data.
    // 4 COMPLETED Finished with all operations.

    //      
    // 1 - state=1, timeout, abort, state=4
    // 2 - state=1, state=2, state=4
    //    this.request.status -        

    if ( this.request.readyState == 4 ) // The request is complete
    {
        if ( this.timeout_timer )
        {
            vk_clearTimeout( this.timeout_timer );
            this.timeout_timer = undefined;
        }

        var is_status_available = false;

        this.in_progress = false;

        try
        {
            if ( this.request.status != undefined )
                is_status_available = true;
        }
        catch (e) {}

        if ( is_status_available && ( this.request.status == 200 ) ) // HTTP status 200 - OK
        {
            var text = this.request.responseText;
            this.StopObserve_();
            vk_global.request_queue.OnRequestFinished();
            this.on_complete( text );
        }
        else
        {
            this.vk_Log( 8, "vk_HTMLProvider", "OnReadyStateChange(), Error loading page" );

            this.StopObserve_();
            vk_global.request_queue.OnRequestFinished();
            this.on_failure();
        }
    }
}

vk_HTMLProvider.prototype.OnTimeout = function()
{
    this.vk_Log( 8, "vk_HTMLProvider", "OnTimeout()" );

    this.Abort();
}

vk_HTMLProvider.prototype.OnError = function( event )
{
    this.vk_Log( 8, "vk_HTMLProvider", "OnError()" );
}

vk_HTMLProvider.prototype.OnLoad = function( event )
{
    this.vk_Log( 8, "vk_HTMLProvider", "OnLoad()" );
}

vk_HTMLProvider.prototype.OnProgress = function( event )
{
    this.vk_Log( 8, "vk_HTMLProvider", "OnProgress()"  );
}
