function vk_GUI()
{
}

vk_GUI.prototype.Init = function()
{
    this.observer_service = vk_global.observer_service;

    this.observer_service.addObserver( this, "vk_State", false );
    this.observer_service.addObserver( this, "vk_Info", false );
    this.observer_service.addObserver( this, "vk_Info-Updated", false );
    this.observer_service.addObserver( this, "vk_Feed2-UpdatingStarted", false );
    this.observer_service.addObserver( this, "vk_Feed2-UpdatingProgress", false );
    this.observer_service.addObserver( this, "vk_Feed2-UpdatingFinished", false );

    this.initialized = true;

    this.is_device = (typeof(widget) != "undefined");

    // Only for non device mode
    this.design_orientation = "portrait";
    this.design_type = "big";

    this.small_priority =
    [
        "messages",
        "wall",
        "friends",
        "meetings",
        "groups",
        "photos",
        //"videos",
        "notes",
        "gifts",
        "offers",
        "opinions",
        "questions"
    ];
    this.big_priority =
    [
        "messages",
        "wall",
        "friends",
        "meetings",
        "groups",
        "photos",
        //"videos",
        "notes",
        "gifts",
        "offers",
        "opinions",
        "questions",
        "news", // Added anyway
        "bookmarks"
    ];

    if ( !this.is_device )
        this.UpdateDesignNonDevice_();

    //this.AddTestCss_();

    this.UpdateDesign_();
    this.UpdateHeaderButtons_();
    this.UpdateFooter_();
    this.UpdateLoginLabel_();
    this.UpdateStatusLabel_();

    vk_global.info.NotifyAll();
    this.UpdateBigInfoButtons_();
    this.UpdateSmallInfoButtons_();
}

vk_GUI.prototype.Destroy = function()
{
    this.initialized = false;

    this.observer_service.removeObserver( this, "vk_State" );
    this.observer_service.removeObserver( this, "vk_Info" );
    this.observer_service.removeObserver( this, "vk_Info-Updated" );
    this.observer_service.removeObserver( this, "vk_Feed2-UpdatingStarted" );
    this.observer_service.removeObserver( this, "vk_Feed2-UpdatingProgress" );
    this.observer_service.removeObserver( this, "vk_Feed2-UpdatingFinished" );

    this.observer_service = undefined;
}

vk_GUI.prototype.observe = function( subject, topic, data )
{
    if ( !this.initialized ) return;

    if ( topic == "vk_Info" )
    {
        if ( data == "user" )
            this.UpdateInfoName_();
        else if ( data == "status" )
            this.UpdateInfoStatus_();
        else
            this.UpdateInfoButton_( data );
    }
    else if ( topic == "vk_Info-Updated" )
    {
        this.UpdateHeaderButtons_();
        this.UpdateFooter_();
        this.UpdateBigInfoButtons_();
        this.UpdateSmallInfoButtons_();
    }
}

vk_GUI.prototype.AddTestCss_ = function()
{
    var style;

    style = document.createElement( "link" );
    style.rel = "stylesheet";
    style.type = "text/css";
    style.href = "design/vk_test.css";
    document.getElementsByTagName( "head" )[0].appendChild( style );
}

vk_GUI.prototype.UpdateDesignNonDevice_ = function()
{
    var small_main_table = document.getElementById( "small_main_table" );

    small_main_table.style.width = "312px";// from documentation - "309px";
    small_main_table.style.height = "82px";// from documentation - "85px";
}

vk_GUI.prototype.ChangeDesignType = function()
{
    if ( this.is_device ) return;

    if ( this.design_type == "big" )
        this.design_type = "small";
    else if ( this.design_type == "small" )
        this.design_type = "big";

    this.UpdateDesignType_();
}

vk_GUI.prototype.ChangeDesignOrientation = function()
{
    if ( this.is_device ) return;

    if ( this.design_orientation == "landscape" )
        this.design_orientation = "portrait";
    else if ( this.design_orientation == "portrait" )
        this.design_orientation = "landscape";

    this.UpdateDesignOrientation_();
}

vk_GUI.prototype.UpdateDesign_ = function()
{
    this.UpdateDesignType_();
    this.UpdateDesignOrientation_();
}

vk_GUI.prototype.UpdateDesignType_ = function()
{
    var big_main = document.getElementById( "big_main" );
    var small_main = document.getElementById( "small_main" );

    if ( this.IsHomeScreen_() )
    {
        big_main.style.display = "none";
        small_main.style.display = "block";
        this.UpdateLogin( false );
        this.UpdateStatus( false );
    }
    else
    {
        small_main.style.display = "none";
        big_main.style.display = "block";
    }
}

vk_GUI.prototype.UpdateDesignOrientation_ = function()
{
    var big_main_table = document.getElementById( "big_main_table" );
    var login_main = document.getElementById( "login_main" );

    if ( !this.is_device )
    {
        if ( this.IsLandscape_() )
        {
            big_main_table.style.width = "502px";
            big_main_table.style.height = "360px";
        }
        else
        {
            big_main_table.style.width = "360px";
            big_main_table.style.height = "579px";
        }

        login_main.style.width = big_main_table.style.width;
        login_main.style.height = big_main_table.style.height;
        status_main.style.width = big_main_table.style.width;
        status_main.style.height = big_main_table.style.height;
    }

    for ( var i = 0, ielem; ielem = document.styleSheets[i]; i++ )
    {
        if ( ielem.href.indexOf( "vk_big.css" ) != -1 )
        {
            for ( var j = 0, jelem; jelem = ielem.cssRules[j]; j++ )
            {
                if ( jelem.selectorText == "#big_main .button_middle" )
                {
                    if ( this.IsLandscape_() )
                        jelem.style.height = "82px";
                    else
                        jelem.style.height = "115px";
                    break;
                }
            }
            break;
        }
    }

    this.UpdateInfoName_();
    this.UpdateBigInfoButtons_();
}

vk_GUI.prototype.UpdateInfoName_ = function()
{
    var big_header_name = document.getElementById( "big_header_name" );
    var big_subheader_name = document.getElementById( "big_subheader_name" );
    var small_header_name = document.getElementById( "small_header_name" );

    if ( vk_global.info.user.extended_complete )
    {
        if ( this.IsLandscape_() )
        {
            big_header_name.innerHTML = this.TruncName_( vk_global.info.user.name );
            big_subheader_name.innerHTML = "";
        }
        else
        {
            big_subheader_name.innerHTML = this.TruncName_( vk_global.info.user.name );
            big_header_name.innerHTML = "";
        }
        small_header_name.innerHTML = this.TruncName_( vk_global.info.user.name );
    }
    else
    {
        big_header_name.innerHTML = "";
        big_subheader_name.innerHTML = "";
        small_header_name.innerHTML = "";
    }
}

vk_GUI.prototype.UpdateInfoButton_ = function( item )
{
    var big_button_elem = document.getElementById( "big_" + item );
    var big_text_elem = document.getElementById( "big_text_" + item );
    var small_text_elem = document.getElementById( "small_text_" + item );

    if ( big_button_elem )
    {
        var text = vk_locale.buttons[item];

        if ( ("count" in vk_global.info[item]) && (vk_global.info[item].count > 0) )
            text += " (<b>" + vk_global.info[item].count + "</b>)";

        big_text_elem.innerHTML = text;
        if ( small_text_elem )
            small_text_elem.innerHTML = text;
    }
}

vk_GUI.prototype.UpdateInfoStatus_ = function()
{
    var space = document.getElementById( "big_footer_status_text" );

    if ( (vk_global.info.status.updated_at != 0) && vk_global.info.status.extended_complete )
        space.innerHTML = vk_global.info.status.text;
    else
        space.innerHTML = "";
}

vk_GUI.prototype.UpdateBigInfoButtons_ = function()
{
    var big_visible_count = 0;
    var big_visible = new Object();
    var big_available = new Array();

    for ( var i = 0, item; item = this.big_priority[i]; i++ )
    {
        if ( "enabled" in vk_global.info[item] )
        {
            if ( vk_global.info[item].enabled )
                big_available.push( item );
        }
        else
            big_available.push( item );
    }

    if ( !vk_global.info.news.enabled )
    {
        big_visible.news = undefined;
        big_visible_count++;
    }

    for ( var i = 0, item; (item = big_available[i]) && (big_visible_count < 12); i++ )
    {
        if ( !("enabled" in vk_global.info[item]) || vk_global.info[item].enabled )
        {
            big_visible[item] = undefined;
            big_visible_count++;
        }
    }

    for ( var i = 0, visible_count = 0, item; item = this.big_priority[i]; i++ )
    {
        var big_button_elem = document.getElementById( "big_" + item );
        var big_br_elem = document.getElementById( "big_br_" + (i + 1) );

        if ( item in big_visible )
        {
            big_button_elem.style.display = "inline-block";
            visible_count++;
        }
        else
            big_button_elem.style.display = "none";

        if (
            (i < 9)
            && (
                (this.IsLandscape_() && (visible_count == 4))
                || (!this.IsLandscape_() && (visible_count == 3))
            )
        )
        {
            big_br_elem.style.display = "inline";
            visible_count = 0;
        }
        else
            big_br_elem.style.display = "none";
    }
}

vk_GUI.prototype.UpdateSmallInfoButtons_ = function()
{
    var small_visible_count = 0;
    var small_visible = new Object();
    var small_available = new Array();

    for ( var i = 0, item; item = this.small_priority[i]; i++ )
    {
        if ( "enabled" in vk_global.info[item] )
        {
            if ( vk_global.info[item].enabled )
                small_available.push( item );
        }
        else
            small_available.push( item );
    }

    for ( var i = 0, item; (item = small_available[i]) && (small_visible_count < 3); i++ )
    {
        if (
            ("count" in vk_global.info[item])
            && (vk_global.info[item].count > 0)
            && (
                !("enabled" in vk_global.info[item])
                || vk_global.info[item].enabled
            )
        )
        {
            small_visible[item] = undefined;
            small_visible_count++;
        }
    }

    if ( small_visible_count < 3 )
    {
        for ( var i = 0, item; (item = small_available[i]) && (small_visible_count < 3); i++ )
        {
            if ( !(item in small_visible) )
            {
                small_visible[item] = undefined;
                small_visible_count++;
            }
        }
    }

    for ( var i = 0, item; item = this.small_priority[i]; i++ )
    {
        var small_button_elem = document.getElementById( "small_" + item );

        if ( item in small_visible )
            small_button_elem.style.display = "inline-block";
        else
            small_button_elem.style.display = "none";
    }
}

vk_GUI.prototype.UpdateHeaderButtons_ = function()
{
    var change_user = document.getElementById( "big_header_change_user" );
    var change_user_text = document.getElementById( "big_header_change_user_text" );

    if ( vk_global.info.user.id == -1 )
    {
        change_user_text.innerHTML = vk_locale.header.login;
        change_user.setAttribute( "onclick", "vk_gui.OnLogin()" )
    }
    else
    {
        change_user_text.innerHTML = vk_locale.header.change_user;
        change_user.setAttribute( "onclick", "vk_gui.OnChangeUser()" )
    }
}

vk_GUI.prototype.UpdateFooter_ = function()
{
    var turn_on_checkbox = document.getElementById( "big_turn_on_checkbox" );
    var turn_on_text = document.getElementById( "big_turn_on_text" );
    var space = document.getElementById( "big_footer_space" );

    turn_on_checkbox.setAttribute( "class", (vk_global.prefs.behavior.turn_on ? "checkbox_on" : "checkbox_off") );
    turn_on_text.innerHTML = vk_locale.footer.turn_on;
    //space.innerHTML += " " + vk_global.info.user.id;
}

vk_GUI.prototype.UpdateLoginLabel_ = function()
{
    var email_label = document.getElementById( "login_email_label" );
    var password_label = document.getElementById( "login_password_label" );
    var anothers_computer_label = document.getElementById( "login_anothers_computer_label" );
    var captcha_code_label = document.getElementById( "login_captcha_code_label" );
    var button_login_text = document.getElementById( "login_button_login_text" );
    var button_cancel_text = document.getElementById( "login_button_cancel_text" );

    email_label.innerHTML = vk_locale.login.email;
    password_label.innerHTML = vk_locale.login.password;
    anothers_computer_label.innerHTML = vk_locale.login.anothers_computer;
    captcha_code_label.innerHTML = vk_locale.login.captcha_code;
    button_login_text.innerHTML = vk_locale.login.login;
    button_cancel_text.innerHTML = vk_locale.login.cancel;
}

vk_GUI.prototype.UpdateLogin = function( show )
{
    //var fade_main = document.getElementById( "fade_main" );
    var login_main = document.getElementById( "login_main" );

    if ( show )
    {
        //fade_main.style.display = "block !important";
        login_main.style.display = "block !important";

        this.ShowLoginFields( 1 );
        this.UpdateLoginProgress( false );
    }
    else
    {
        //fade_main.style.display = "none !important";
        login_main.style.display = "none !important";
    }
}

vk_GUI.prototype.UpdateAnothersComputer_ = function()
{
    var anothers_computer = document.getElementById( "login_anothers_computer" );
    var anothers_computer_checkbox = document.getElementById( "login_anothers_computer_checkbox" );

    anothers_computer_checkbox.setAttribute( "class", (anothers_computer.checked ? "checkbox_on" : "checkbox_off") );
}

vk_GUI.prototype.UpdateLoginProgress = function( show )
{
    var progress = document.getElementById( "login_progress" );

    if ( show )
        progress.style.display = "inline-block";
    else
        progress.style.display = "none";
}

vk_GUI.prototype.UpdateStatusLabel_ = function()
{
    var text_label = document.getElementById( "status_text_label" );
    var button_edit_text = document.getElementById( "status_button_edit_text" );
    var button_cancel_text = document.getElementById( "status_button_cancel_text" );

    text_label.innerHTML = vk_locale.status.text;
    button_edit_text.innerHTML = vk_locale.status.edit;
    button_cancel_text.innerHTML = vk_locale.status.cancel;
}

vk_GUI.prototype.UpdateStatus = function( show )
{
    //var fade_main = document.getElementById( "fade_main" );
    var status_main = document.getElementById( "status_main" );
    var status_text = document.getElementById( "status_text" );

    if ( show )
    {
        if ( vk_global.info.status.hash )
        {
            //fade_main.style.display = "block !important";
            status_main.style.display = "block !important";

            if ( vk_global.info.status.text )
                status_text.value = vk_global.info.status.text;
            else
                status_text.value = "";

            this.UpdateStatusProgress( false );
        }
    }
    else
    {
        //fade_main.style.display = "none !important";
        status_main.style.display = "none !important";
    }
}

vk_GUI.prototype.UpdateStatusProgress = function( show )
{
    var progress = document.getElementById( "status_progress" );

    if ( show )
        progress.style.display = "inline-block";
    else
        progress.style.display = "none";
}

vk_GUI.prototype.ShowLoginFields = function( part, param )
{
    var fields_part_1 = document.getElementById( "login_fields_part_1" );
    var fields_part_2 = document.getElementById( "login_fields_part_2" );
    var email = document.getElementById( "login_email" );
    var password = document.getElementById( "login_password" );
    var anothers_computer = document.getElementById( "login_anothers_computer" );
    var captcha_sid = document.getElementById( "login_captcha_sid" );
    var captcha_code = document.getElementById( "login_captcha_code" );
    var captcha_image = document.getElementById( "login_captcha_image" );

    captcha_sid.value = "";
    captcha_code.value = "";
    captcha_image.src = "";

    if ( part == 1 )
    {
        fields_part_1.style.display = "block";
        fields_part_2.style.display = "none";
        // param -> is continue login
        if ( !param )
        {
            email.value = "";
            password.value = "";
            anothers_computer.checked = false;
        }
        this.UpdateAnothersComputer_();
    }
    else
    {
        fields_part_1.style.display = "none";
        fields_part_2.style.display = "block";
        // param -> new captcha sid
        captcha_sid.value = param;
        captcha_image.src = vk_global.prefs.urls.main + vk_global.prefs.urls.captcha + param;
    }
}

vk_GUI.prototype.OnResize = function()
{
    if ( !this.is_device ) return;

    this.UpdateDesign_();
}

vk_GUI.prototype.OnLogin = function()
{
    this.UpdateLogin( true );
}

vk_GUI.prototype.OnChangeUser = function()
{
    ClearCookies();
    vk_global.state.UpdateLoginState();

    this.UpdateLogin( true );
}

vk_GUI.prototype.OnAnothersComputer = function()
{
    var anothers_computer = document.getElementById( "login_anothers_computer" );

    anothers_computer.checked = !anothers_computer.checked;
    this.UpdateAnothersComputer_();
}

vk_GUI.prototype.OnLoginLogin = function()
{
    var email = document.getElementById( "login_email" ).value;
    var password = document.getElementById( "login_password" ).value;
    var anothers_computer = document.getElementById( "login_anothers_computer" ).checked;
    var captcha_sid = document.getElementById( "login_captcha_sid" ).value;
    var captcha_code = document.getElementById( "login_captcha_code" ).value;

    Login( email, password, anothers_computer, captcha_sid, captcha_code );
}

vk_GUI.prototype.OnLoginCancel = function()
{
    this.UpdateLogin( false );
}

vk_GUI.prototype.OnLoginFailure = function()
{
    alert( vk_locale.login.failed );
}

vk_GUI.prototype.OnStatus = function()
{
    this.UpdateStatus( true );
}

vk_GUI.prototype.OnStatusEdit = function()
{
    var text = document.getElementById( "status_text" ).value;

    EditStatus( vk_global.info.status.hash, text );
}

vk_GUI.prototype.OnStatusCancel = function()
{
    this.UpdateStatus( false );
}

vk_GUI.prototype.OnStatusFailure = function()
{
    alert( vk_locale.status.failed );
}

vk_GUI.prototype.OpenUrl = function( url )
{
    if ( this.is_device )
    {
/*
            var sysinfo = document.embeds[0];
            vk_dump( "sysinfo=" + sysinfo );
            vk_dump( "===================================================================================" + sysinfo.startvibra );
            vk_dump( "sysinfo.vibraminintensity=" + sysinfo.vibraminintensity );
            vk_dump( "sysinfo.vibramaxintensity=" + sysinfo.vibramaxintensity );
            vk_dump( "sysinfo.vibramaxduration=" + sysinfo.vibramaxduration );
            vk_dump( "sysinfo.vibrasettings=" + sysinfo.vibrasettings );
            //vk_inspect( sysinfo );
            var duration = sysinfo.vibramaxduration;
            var intensity = sysinfo.vibramaxintensity;
            //sysinfo.startvibra( duration, intensity );
            sysinfo.startvibra( 10, 10 );
            vk_dump( "-10-10-------------------------------------------------------------------------------------" );
        try
        {
        }
        catch (e) {}
*/
        widget.openURL( url );
    }
    else
        window.open( url, "_self" );
}

vk_GUI.prototype.GetScreenSize_ = function()
{
    var width = window.innerWidth;
    var height = window.innerHeight;

    return {
        width: width,
        height: height
    };
}

vk_GUI.prototype.IsLandscape_ = function()
{
    if ( this.is_device )
    {
        var size = this.GetScreenSize_();
        return ( size.width > size.height );
    }
    else
        return ( this.design_orientation == "landscape" );
}

vk_GUI.prototype.IsHomeScreen_ = function()
{
    //document.getElementById( "big_footer_space" ).innerHTML += " " + this.GetScreenSize_().width + "x" + this.GetScreenSize_().height;
/*
    vk_dump(
        "innerWidth=" + window.innerWidth
        + ", innerHeight=" + window.innerHeight
        + ", outerWidth=" + window.outerWidth
        + ", outerHeight=" + window.outerHeight
        + ", width=" + document.width
        + ", height=" + document.height
    );
*/

    if ( this.is_device )
        return ( this.GetScreenSize_().height < 150 );
    else
        return ( this.design_type == "small" );
}

vk_GUI.prototype.TruncName_ = function( str )
{
    if ( str.length <= vk_global.prefs.gui.max_name_length )
        return str;
    else
        return str.substr( 0, vk_global.prefs.gui.max_name_length ) + "...";

}

vk_GUI.prototype.TruncStatus_ = function( str )
{
    if ( str.length <= vk_global.prefs.gui.max_status_length )
        return str;
    else
        return str.substr( 0, vk_global.prefs.gui.max_status_length ) + "...";

}
